"""
Kreveik is a package that is meant to provide access to a set of tools 
about 
"""

import classes
import numpy as num
import matplotlib.pyplot as plt
import network
import probes
import copy
import genetic
import family
import parallel
import logging
logging.basicConfig(level=logging.INFO)

__all__ = ['classes','family','genetic','network','probes']

